@@ -6,7 +6,7 @@
  *  Written by Drew Sullivan.
  *  Rewritten by Mike Jagdis.
  *
- * $Id: ioctl.c,v 1.51 1997/05/10 14:45:53 jaggy Exp $
+ * $Id: ioctl.c,v 1.54 1997/11/05 09:32:19 jaggy Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/ioctl.c,v $
  */
 #include <linux/config.h>
@@ -81,12 +81,8 @@ do_ioctl(struct pt_regs *regs, int fd, unsigned long ioctl_num, void *arg)
 	unsigned int class = ioctl_num >> 8;
 
 	switch (class) {
-		case 0:
-			/* SCO ioctls on the pseudo NFS device probably.
-			 * These get passed off to the socksys driver via
-			 * the standard handler.
-			 */
-			return SYS(ioctl)(fd, ioctl_num, arg);
+		case 0: /* SCO ioctls on the pseudo NFS device probably. */
+			return ibcs_ioctl_socksys(fd, ioctl_num, arg);
 
 		case 'A': /* ??? SCO console keyboard stuff? */
 			return -EINVAL;
@@ -111,7 +107,8 @@ do_ioctl(struct pt_regs *regs, int fd, unsigned long ioctl_num, void *arg)
 			return ibcs_ioctl_termios(fd, ioctl_num & 0xFF, arg);
 
 		case 'C':
-			return ibcs_ioctl_console(fd, ioctl_num & 0xFF, arg);
+		case 'c':
+			return ibcs_ioctl_console(fd, ioctl_num, arg);
 
 		case ('i' << 16) | ('C' << 8):	/* iBCS2 POSIX */
 			return ibcs_ioctl_video(fd, ioctl_num & 0xFF, arg);
@@ -730,10 +727,14 @@ static int ibcs_ioctl_termio(int fd, unsigned int func, void *arg) {
 
 static int ibcs_ioctl_console(int fd, unsigned int func, void *arg) {
 	switch(func) {
-		case 4: /* _TTYDEVTYPE */
-			/* ***BUG*** if on console then 1, if pseudo tty
-			 * then 2
+		case 0x6301: /* CONS_CURRENT: Get display adapter type */
+		case 0x6302: /* CONS_GET: Get display mode setting */
+			/* Always error so the application never tries
+			 * anything overly fancy on the console.
 			 */
+			return -EINVAL;
+		case 0x4304: /* _TTYDEVTYPE */
+			/* If on console then 1, if pseudo tty then 2 */
 			return 2;
 	}
 	printk(KERN_ERR "iBCS: console ioctl %d unsupported\n", func);
@@ -826,8 +827,21 @@ static int ibcs_ioctl_termiox(int fd, unsigned int func, void *arg) {
 static int ibcs_ioctl_stream(struct pt_regs *regs,
 	int fd, unsigned int func, void *arg)
 {
+	struct file *filep;
+	struct inode *ino;
 	int error;
 
+	if (!(filep = current->FD[fd]) || !(ino = filep->f_inode))
+		return -EBADF;
+
+	if (MAJOR(ino->i_rdev) == socksys_major) {
+		error = ibcs_socksys_fd_init(fd, 0, NULL, 0);
+		if (error)
+			return error;
+		filep = current->FD[fd];
+		ino = filep->f_inode;
+	}
+
 	switch (func) {
 		case 001: /* I_NREAD */
 			error = verify_area(VERIFY_WRITE,
@@ -835,9 +849,9 @@ static int ibcs_ioctl_stream(struct pt_regs *regs,
 			if (error)
 				return error;
 #ifdef EMU_XTI
-			if (!current->FD[fd])
+			if (!filep || !ino)
 				return -EBADF;
-			if (current->FD[fd]->f_inode->i_sock
+			if (ino->i_sock
 			&& Priv(fd) && Priv(fd)->pfirst) {
 				put_fs_long(Priv(fd)->pfirst->length, arg);
 				return 1; /* at least 1... (FIXME) */
@@ -946,10 +960,10 @@ static int ibcs_ioctl_stream(struct pt_regs *regs,
 				return error;
 			memcpy_fromfs(&buf, arg, sizeof(buf));
 
-			if (!current->FD[fd])
+			if (!filep)
 				return -EBADF;
 
-			if (current->FD[fd]->f_inode->i_sock
+			if (ino->i_sock
 			&& Priv(fd) && Priv(fd)->pfirst
 			&& (!buf.flags || buf.flags == Priv(fd)->pfirst->pri)) {
 				int l = buf.ctl.maxlen <= Priv(fd)->pfirst->length
