@@ -11,7 +11,7 @@
  *  Please consider this closely if you plan on changing this mode.
  *  -- Al Longyear
  *
- * $Id: signal.c,v 1.23 1995/11/29 13:00:39 mike Exp $
+ * $Id: signal.c,v 1.25 1997/02/18 22:05:38 jaggy Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/signal.c,v $
  */
 
@@ -44,7 +44,6 @@
 #include <ibcs/abi4.h>
 #include <ibcs/map.h>
 
-#include <signal.h>
 
 #define SIG_HOLD	((__sighandler_t)2)	/* hold signal */
 
@@ -66,10 +65,14 @@ typedef void (*pfn) (void);     /* Completion function */
  *  defines the stack frame.
  */
 
-#define SIGNAL_NUMBER    get_fs_long (&((unsigned long *) regs->esp) [1])
+#define SIGNAL_NUMBER    get_syscall_parameter (regs, 0)
 #define HIDDEN_PARAM     (SIGNAL_NUMBER & ~0xFF)
-#define SECOND_PARAM     get_fs_long (&((unsigned long *) regs->esp) [2])
+#define SECOND_PARAM     get_syscall_parameter (regs, 1)
+#ifdef __sparc__
+#define THIRD_PARAM      get_syscall_parameter (regs, 2)
+#else /* __sparc__ */
 #define THIRD_PARAM      ((unsigned long) regs->edx)
+#endif /* __sparc__ */
 
 /* Return a mask that includes SIG only.  */
 #define __sigmask(sig)	(1 << ((sig) - 1))
@@ -135,8 +138,7 @@ inline int ibcs_signo (struct pt_regs *regs, int *sig)
 	int    value = ibcs_mapsig(SIGNAL_NUMBER & 0xFF);
 
 	if (value == -1) {
-		regs->eax     = iABI_errors (EINVAL);
-		regs->eflags |= 1;
+		set_error (regs, iABI_errors (EINVAL));
 		return 0;
 	}
 
@@ -174,10 +176,9 @@ void ibcs_sig_handler (struct pt_regs * regs, int sig,
 	FROM_KERNEL (old_fs);
 
 	if (answer < 0) {
-		regs->eax     = iABI_errors (-answer);
-		regs->eflags |= 1;
+		set_error (regs, iABI_errors (-answer));
 	} else
-		regs->eax     = (int) oact.sa_handler;
+		set_result (regs, (int) oact.sa_handler);
 }
 
 /*
@@ -232,8 +233,7 @@ int ibcs_sigset (struct pt_regs * regs)
 			FROM_KERNEL (old_fs);
 
 			if (answer < 0) {
-				regs->eax     = iABI_errors (-answer);
-				regs->eflags |= 1;
+				set_error (regs, iABI_errors (-answer));
 			}
 		}
 	}
@@ -262,8 +262,7 @@ void ibcs_sighold (struct pt_regs * regs)
 	FROM_KERNEL (old_fs);
 
 	if (answer < 0) {
-		regs->eax     = iABI_errors (-answer);
-		regs->eflags |= 1;
+		set_error (regs, iABI_errors (-answer));
 	}
 }
 
@@ -293,8 +292,7 @@ void ibcs_sigrelse (struct pt_regs * regs)
 	FROM_KERNEL (old_fs);
 
 	if (answer < 0) {
-		regs->eax     = iABI_errors (-answer);
-		regs->eflags |= 1;
+		set_error (regs, iABI_errors (-answer));
 	}
 }
 
@@ -324,8 +322,7 @@ void ibcs_sigignore (struct pt_regs * regs)
 	FROM_KERNEL (old_fs);
 
 	if (answer < 0) {
-		regs->eax     = iABI_errors (-answer);
-		regs->eflags |= 1;
+		set_error (regs, iABI_errors (-answer));
 	}
 }
 
@@ -344,6 +341,9 @@ void ibcs_sigpause (struct pt_regs * regs)
 	sigset_t   newset;
 	int	sig, answer;
 
+#ifdef __sparc__
+	printk(KERN_ERR "Sparc/iBCS: sigpause not yet implemented\n");
+#else
 	if (!ibcs_signo(regs, &sig))
 		return;
 
@@ -361,6 +361,7 @@ void ibcs_sigpause (struct pt_regs * regs)
 		regs->eax = iABI_errors(-answer);
 		regs->eflags |= 1;
 	}
+#endif
 }
 
 /*
@@ -389,9 +390,12 @@ int ibcs_sigfunc (struct pt_regs * regs)
 	}
 #endif
 
+#ifdef __sparc__
+	set_result (regs, 0);
+#else /* __sparc__ */
 	regs->eflags &= ~1;
 	regs->eax     = 0;
-
+#endif /* __sparc__ */
 	switch (sig_type) {
 	case 0x0000:
 		ibcs_signal (regs);
@@ -418,8 +422,7 @@ int ibcs_sigfunc (struct pt_regs * regs)
 		break;
 
 	default:
-		regs->eax     = EINVAL;
-		regs->eflags |= 1;
+		set_error (regs, EINVAL);
 
 #ifdef IBCS_TRACE
 		if ((ibcs_trace & (TRACE_SIGNAL | TRACE_SIGNAL_F))
@@ -436,7 +439,7 @@ int ibcs_sigfunc (struct pt_regs * regs)
 #ifdef IBCS_TRACE
 	if ((ibcs_trace & (TRACE_SIGNAL | TRACE_SIGNAL_F))
 	|| ibcs_func_p->trace) {
-		printk(KERN_DEBUG "iBCS2 returns %ld\n", regs->eax);
+		printk(KERN_DEBUG "iBCS2 returns %ld\n", get_result (regs));
 	}
 #endif
 	return 0;
@@ -664,7 +667,7 @@ abi_sigprocmask(int how, sigset_t *abinset, sigset_t *abioset)
 	return error;
 }
 
-
+#ifndef __sparc__
 int abi_sigsuspend(struct pt_regs * regs)
 {
 	sigset_t * set;
@@ -673,16 +676,15 @@ int abi_sigsuspend(struct pt_regs * regs)
 
 #if defined(EMU_BSD) && defined(PER_BSD)
 	if (current->personality == PER_BSD) {
-		oldset = get_fs_long(((unsigned long *)regs->esp)+1);
+		oldset = get_syscall_parameter (regs, 0);
 	} else
 #endif
 	{
-		set = (sigset_t *)get_fs_long(((unsigned long *)regs->esp)+1);
-		if ((error = verify_area(VERIFY_READ, set, sizeof(long))))
+		set = (sigset_t *)get_syscall_parameter (regs, 0);
+		if ((error = verify_area(VERIFY_READ, set, sizeof(sigset_t))))
 			return error;
 		oldset = get_fs_long ((unsigned long *) set);
 	}
-
 #ifdef INIT_MM
 	newset = map_bitvec(oldset,
 		current->exec_domain->signal_map);
@@ -696,6 +698,12 @@ int abi_sigsuspend(struct pt_regs * regs)
 		printk("iBCS: sigsuspend oldset, newset = %lx %lx\n",
 			oldset, newset);
 #endif
+	{
+#if 0
+	    extern do_sigpause(unsigned int, struct pt_regs *);
+	    return do_sigpause(newset, regs);
+#endif
+	}
 	return SYS(sigsuspend)(0, oldset,
 			newset, regs->esi, regs->edi,
 			regs->ebp, regs->eax,
@@ -704,3 +712,4 @@ int abi_sigsuspend(struct pt_regs * regs)
 			regs->eip, regs->cs, regs->eflags,
 			regs->esp, regs->ss);
 }
+#endif /* __sparc__ */
