@@ -3,7 +3,7 @@
  *
  *  Copyright 1995, 1996  Mike Jagdis (jaggy@purplet.demon.co.uk)
  *
- * $Id: timod.c,v 1.21 1996/07/26 12:03:40 mike Exp $
+ * $Id: timod.c,v 1.22 1997/11/05 09:32:20 jaggy Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/timod.c,v $
  */
 
@@ -1328,14 +1328,21 @@ timod_ioctl(struct pt_regs *regs,
 int
 ibcs_ioctl_sockmod(int fd, unsigned int func, void *arg)
 {
-	int error;
+	struct file *filep;
 	struct inode *ino;
+	int error;
 
-	if (!current->FD[fd]
-	|| !(ino = current->FD[fd]->f_inode)
-	|| !ino->i_sock)
+	if (!(filep = current->FD[fd]) || !(ino = filep->f_inode))
 		return TBADF;
 
+	if (MAJOR(ino->i_rdev) == socksys_major) {
+		error = ibcs_socksys_fd_init(fd, 0, NULL, 0);
+		if (error)
+			return error;
+		filep = current->FD[fd];
+		ino = filep->f_inode;
+	}
+
 	switch (func) {
 #ifdef __sparc__
             	case 110: { /* SI_GETUDATA -- Solaris */
@@ -1352,7 +1359,7 @@ ibcs_ioctl_sockmod(int fd, unsigned int func, void *arg)
 			if (ibcs_trace & TRACE_STREAMS)
 				printk(KERN_DEBUG "iBCS: [%d] %lx new_getudata\n",
 					current->pid,
-					(unsigned long)current->FD[fd]);
+					(unsigned long)filep);
 #endif
 			error = verify_area(VERIFY_WRITE, it, sizeof(*it));
 			if (error)
@@ -1395,7 +1402,7 @@ ibcs_ioctl_sockmod(int fd, unsigned int func, void *arg)
 			if (ibcs_trace & TRACE_STREAMS)
 				printk(KERN_DEBUG "iBCS: [%d] %lx getudata\n",
 					current->pid,
-					(unsigned long)current->FD[fd]);
+					(unsigned long)filep);
 #endif
 			error = verify_area(VERIFY_WRITE, it, sizeof(*it));
 			if (error)
